from pydantic import BaseModel, Field
from typing import List, Dict, Optional
from datetime import datetime


class ChecklistV2(BaseModel):
    id: int
    name: str
    created_by_id: int
    line_items: list


class MemberV2(BaseModel):
    name: str
    first_name: Optional[str] = Field(default=None)
    last_name: Optional[str] = Field(default=None)
    employee_identification_number: str
    fax: Optional[str] = Field(default=None)
    email: Optional[str] = Field(default=None)
    phone_number: int
    # time_zone: 
    address: Optional[str] = Field(default=None)
    city: Optional[str] = Field(default=None)
    state: Optional[str] = Field(default=None)
    country: Optional[str] = Field(default=None)
    zip_code: Optional[str] = Field(default=None)
    description: Optional[str] = Field(default=None)
    display_picture_url: Optional[str] = Field(default=None)
    department: Optional[str] = Field(default=None)

class WorkOrderV2(BaseModel):
    approver_id: Optional[int] = Field(default=None)
    assigned_to_id: Optional[int] = Field(default=None)
    assigned_to_type: str
    associated_checklists: list
    base_cost: float
    completed_on: Optional[str] = Field(default=None)
    create_one_task_for_all_items: bool
    create_recurring_service_zendesk_tickets: bool
    created_at: str
    created_by_id: Optional[int] = Field(default=None)
    creation_source: Optional[str] = Field(default=None)
    custom_fields: Optional[List[dict]]
    description: Optional[str] = Field(default=None)
    display_next_service_immediately: bool
    due_date: Optional[datetime] = Field(default=None)
    expected_start_date: Optional[datetime] = Field(default=None)
    id: int
    inventory_cost: float
    inventory_cost_method: Optional[str] = Field(default=None)
    is_item_component: bool
    is_triage: bool
    location_id: Optional[int] = Field(default=None)
    mark_items_unavailable: bool
    preventive_maintenance: bool
    priority: str
    project_id: Optional[int] = Field(default=None)
    recurrence_based_on_completion_date: bool
    recurrence_task_id: Optional[int | None]
    repeat_every_basis: Optional[str] = Field(default=False)
    repeat_every_value: int
    repetition_end_date: Optional[str] = Field(default=None)
    repetition_starting: Optional[str] = Field(default=None)
    requested_by_id: Optional[int] = Field(default=None)
    require_approval_from_reviewer: bool
    reviewer_id: Optional[int] = Field(default=None)
    service_for_sub_groups_only: bool
    service_type_id: Optional[int] = Field(default=None)
    shipping_address_id: Optional[int] = Field(default=None)
    start_work_on_all_assets: bool
    started_on: Optional[str] = Field(default=None)
    state: str
    supervisor_id: Optional[int] = Field(default=None)
    task_type: str
    task_type_id: Optional[int] = Field(default=None)
    template_id: Optional[int] = Field(default=None)
    time_spent: float
    # time_to_respond:
    time_to_start: int
    title: str
    total_cost: float
    track_progress: float
    updated_at: str
    warranty: Optional[bool] = Field(default=False)
    work_logs_cost: float
    work_type_name: Optional[str] = Field(default=None)
    zendesk_ticket_id: Optional[int]



"""
This module contains functions to interact with the checklist v2 API in EZOfficeInventory.
"""

import os
from typing import Literal, Optional
from datetime import date, datetime
import requests
from pprint import pprint

from ezoff._auth import Decorators
from ezoff._helpers import _basic_retry, _fetch_page
from .exceptions import *
from .data_model import *

@Decorators.check_env_vars
def get_checklists_v2_pd() -> Dict[str, ChecklistV2]:
    """
    Get all checklists from EZ Office V2 API Call.
    Returns dictionary of pydantic objects keyed by checklist id.
    """
    checklist_dict = get_checklists_v2()
    checklists = {}

    for c in checklist_dict:
        checklist = checklist_dict[c]
        try:
            checklists[checklist['id']] = ChecklistV2(**checklist)

        except Exception as e:
            print(str(e))
            pprint(checklist)
            exit(0)

    return checklists


@Decorators.check_env_vars
def get_checklists_v2() -> dict:
    """
    Get all checklists from EZ Office. V2 API Call.

    https://pepsimidamerica.ezofficeinventory.com/api/v2/checklists?page=1&per_page=100
    """

    url = os.environ["EZO_BASE_URL"] + "/api/v2/checklists"

    page = 1
    per_page = 100
    all_checklists = {}

    while True:
        params = {"page": page, "per_page": per_page}

        try:
            response = _fetch_page(
                url,
                headers={"Authorization": "Bearer " + os.environ["EZO_TOKEN"]},
                params=params,
            )
            response.raise_for_status()

        except requests.exceptions.HTTPError as e:
            raise Exception(
                f"Error, could not get checklists: {e.response.status_code} - {e.response.content}"
            )
        
        except requests.exceptions.RequestException as e:
            raise Exception(f"Error, could not get checklists: {e}")

        data = response.json()

        if "checklists" not in data:
            raise NoDataReturned(f"No checklists found: {response.content}")

        for checklist in data["checklists"]:
            all_checklists[checklist["id"]] = checklist

        metadata = data["metadata"]

        if "total_pages" not in metadata:
            break

        if page >= metadata["total_pages"]:
            break

        page += 1

    return all_checklists
